@@ -27,13 +27,12 @@ import os
 import signal
 import os.path
 import time
-import signal
 
 # Local
 from base.g import *
 from base import device, utils, models
 from base.codes import *
-from ui_utils import *
+from .ui_utils import *
 
 # PyQt
 try:
@@ -43,7 +42,7 @@ except ImportError:
     log.error("Python bindings for Qt4 not found. Try using --qt3. Exiting!")
     sys.exit(1)
 
-from systrayframe import SystrayFrame
+from .systrayframe import SystrayFrame
 
 # dbus (required)
 try:
@@ -64,9 +63,12 @@ warnings.simplefilter("ignore", DeprecationWarning)
 # pynotify (optional)
 have_pynotify = True
 try:
-    import pynotify
+    import notify2 as pynotify
 except ImportError:
-    have_pynotify = False
+    try:
+        import pynotify
+    except ImportError:
+        have_pynotify = False
 
 
 TRAY_MESSAGE_DELAY = 10000
@@ -74,8 +76,8 @@ HIDE_INACTIVE_DELAY = 5000
 BLIP_DELAY = 2000
 SET_MENU_DELAY = 1000
 MAX_MENU_EVENTS = 10
-UPGRADE_CHECK_DELAY=24*60*60*1000 		#1 day
-CLEAN_EXEC_DELAY=4*60*60*1000 		#4 Hrs
+UPGRADE_CHECK_DELAY=24*60*60*1000        #1 day
+#CLEAN_EXEC_DELAY=4*60*60*1000            #4 Hrs
 
 ERROR_STATE_TO_ICON = {
     ERROR_STATE_CLEAR:        QSystemTrayIcon.Information,
@@ -132,7 +134,7 @@ class DeviceMenu(QMenu):
                 ess = device.queryString(e.event_code, 0)
 
                 a = QAction(QIcon(getStatusListIcon(error_state)[self.index]),
-                                    QString("%1 %2").arg(ess).arg(getTimeDeltaDesc(e.timedate)), self)
+                                    QString("%s %s"%(ess,getTimeDeltaDesc(e.timedate))), self)
 
                 if first:
                     f = a.font()
@@ -170,19 +172,19 @@ class HistoryDevice(QObject):
 
         if back_end == 'hp':
             self.device_type = DEVICE_TYPE_PRINTER
-            self.menu_text = self.__tr("%1 Printer (%2)").arg(self.model).arg(self.id)
+            self.menu_text = self.__tr("%s Printer (%s)"%(self.model,self.id))
 
         elif back_end == 'hpaio':
             self.device_type = DEVICE_TYPE_SCANNER
-            self.menu_text = self.__tr("%1 Scanner (%2)").arg(self.model).arg(self.id)
+            self.menu_text = self.__tr("%s Scanner (%s)"%(self.model,self.id))
 
         elif back_end == 'hpfax':
             self.device_type = DEVICE_TYPE_FAX
-            self.menu_text = self.__tr("%1 Fax (%2)").arg(self.model).arg(self.id)
+            self.menu_text = self.__tr("%s Fax (%s)"%(self.model,self.id))
 
         else:
             self.device_type = DEVICE_TYPE_UNKNOWN
-            self.menu_text = self.__tr("%1 (%2)").arg(self.model).arg(self.id)
+            self.menu_text = self.__tr("%s (%s)"%(self.model,self.id))
 
         self.mq = device.queryModelByURI(self.device_uri)
         self.index = 0
@@ -404,7 +406,8 @@ class SystemTrayApp(QApplication):
                 try:
                     os.waitpid(0, os.WNOHANG)
                 except OSError:
-                    pass
+                   pass
+
             return
             
             
@@ -653,10 +656,10 @@ class SystemTrayApp(QApplication):
                 break
 
             if r:
-                m = ''.join([m, os.read(self.read_pipe, self.fmt_size)])
+                #m = ''.join([m, os.read(self.read_pipe, self.fmt_size)])
+                m = os.read(self.read_pipe, self.fmt_size)
                 while len(m) >= self.fmt_size:
-                    event = device.Event(*struct.unpack(self.fmt, m[:self.fmt_size]))
-
+                    event = device.Event(*[x.rstrip(b'\x00').decode('utf-8') if isinstance(x, bytes) else x for x in struct.unpack(self.fmt, m[:self.fmt_size])])
                     m = m[self.fmt_size:]
                     
                     if event.event_code == EVENT_CUPS_QUEUES_REMOVED or event.event_code == EVENT_CUPS_QUEUES_ADDED:
@@ -677,7 +680,7 @@ class SystemTrayApp(QApplication):
 
                     if self.user_settings.systray_visible in \
                         (SYSTRAY_VISIBLE_SHOW_ALWAYS, SYSTRAY_VISIBLE_HIDE_WHEN_INACTIVE):
-
+                        
                         log.debug("Showing...")
                         self.tray_icon.setVisible(True)
 
@@ -750,16 +753,16 @@ class SystemTrayApp(QApplication):
                                 self.model = models.normalizeModelUIName(model)
 
                                 if back_end == 'hp':
-                                    d = self.__tr("%1 Printer (%2)").arg(model).arg(idd)
+                                    d = self.__tr("%s Printer (%s)"%(model,idd))
 
                                 elif back_end == 'hpaio':
-                                    d = self.__tr("%1 Scanner (%2)").arg(model).arg(idd)
+                                    d = self.__tr("%s Scanner (%s)"%(model,idd))
 
                                 elif back_end == 'hpfax':
-                                    d = self.__tr("%1 Fax (%2)").arg(model).arg(idd)
+                                    d = self.__tr("%s Fax (%s)"%(model,idd))
 
                                 else:
-                                    d = self.__tr("%1 (%2)").arg(model).arg(idd)
+                                    d = self.__tr("%s (%s)"%(model,idd))
 
                             if show_message:
                                 if have_pynotify and pynotify.init("hplip"): # Use libnotify/pynotify
@@ -767,14 +770,16 @@ class SystemTrayApp(QApplication):
                                         (getPynotifyIcon('info'), pynotify.URGENCY_NORMAL))
 
                                     if event.job_id and event.title:
-                                        msg = "%s\n%s: %s\n(%s/%s)" % (unicode(d), desc, event.title, event.username, event.job_id)
+                                        msg = "%s\n%s: %s\n(%s/%s)" % (to_unicode(d), desc, event.title, event.username, event.job_id)
                                         log.debug("Notify: uri=%s desc=%s title=%s user=%s job_id=%d code=%d" %
                                                 (event.device_uri, desc, event.title, event.username, event.job_id, event.event_code))
                                     else:
-                                        msg = "%s\n%s (%s)" % (unicode(d), desc, event.event_code)
+                                        msg = "%s\n%s (%s)" % (to_unicode(d), desc, event.event_code)
                                         log.debug("Notify: uri=%s desc=%s code=%d" % (event.device_uri, desc, event.event_code))
 
                                     n = pynotify.Notification("HPLIP Device Status", msg, icon)
+                                    # CRID: 11833 Debian Traceback error notification exceeded
+                                    n.set_hint('transient', True)
                                     n.set_urgency(urgency)
 
                                     if error_state == ERROR_STATE_ERROR:
@@ -782,10 +787,7 @@ class SystemTrayApp(QApplication):
                                     else:
                                         n.set_timeout(TRAY_MESSAGE_DELAY)
 
-                                    try:
-                                        n.show()
-                                    except:
-                                        log.error("Failed to show notification!")
+                                    n.show()
 
                                 else: # Use "standard" message bubbles
                                     icon = ERROR_STATE_TO_ICON.get(error_state, QSystemTrayIcon.Information)
@@ -793,17 +795,13 @@ class SystemTrayApp(QApplication):
                                         log.debug("Bubble: uri=%s desc=%s title=%s user=%s job_id=%d code=%d" %
                                                 (event.device_uri, desc, event.title, event.username, event.job_id, event.event_code))
                                         self.tray_icon.showMessage(self.__tr("HPLIP Device Status"),
-                                            QString("%1\n%2: %3\n(%4/%5)").\
-                                            arg(d).\
-                                            arg(desc).arg(event.title).\
-                                            arg(event.username).arg(event.job_id),
+                                            QString("%s\n%s: %s\n(%s/%s)"%(d,desc, event.title,event.username,event.job_id)),
                                             icon, TRAY_MESSAGE_DELAY)
 
                                     else:
                                         log.debug("Bubble: uri=%s desc=%s code=%d" % (event.device_uri, desc, event.event_code))
                                         self.tray_icon.showMessage(self.__tr("HPLIP Device Status"),
-                                            QString("%1\n%2 (%3)").arg(d).\
-                                            arg(desc).arg(event.event_code),
+                                            QString("%s\n%s (%s)"%(d,desc,event.event_code)),
                                             icon, TRAY_MESSAGE_DELAY)
 
             else:
@@ -828,7 +826,7 @@ def run(read_pipe):
 
     try:
         app = SystemTrayApp(sys.argv, read_pipe)
-    except dbus.DBusException, e:
+    except dbus.DBusException as e:
         # No session bus
         log.debug("Caught exception: %s" % e)
         sys.exit(1)
